<?php
/**
 * This file is part of the official Amazon Checkout extension for Magento 1.x
 *
 * (c) 2020 creativestyle GmbH. All Rights reserved
 *
 * Distribution of the derivatives reusing, transforming or being built upon
 * this software, is not allowed without explicit written permission granted
 * by creativestyle GmbH
 *
 * @category   Creativestyle
 * @package    Creativestyle_AmazonCheckout
 * @copyright  2020 creativestyle GmbH
 * @author     Marek Stefan <ticket@creativestyle.de>
 */

class Creativestyle_AmazonCheckout_Model_Api_CheckoutSession
    extends Creativestyle_AmazonCheckout_Model_Api_AbstractModel
    implements Creativestyle_AmazonCheckout_Model_Api_CheckoutSessionInterface
{
    const FIELD_TYPE_MAP = array(
        'webCheckoutDetails'        => Creativestyle_AmazonCheckout_Model_Api_WebCheckoutDetails::class,
        'paymentDetails'            => Creativestyle_AmazonCheckout_Model_Api_PaymentDetails::class,
        'merchantMetadata'          => Creativestyle_AmazonCheckout_Model_Api_MerchantMetadata::class,
        'buyer'                     => Creativestyle_AmazonCheckout_Model_Api_Buyer::class,
        'paymentPreferences'        => array(Creativestyle_AmazonCheckout_Model_Api_PaymentPreference::class),
        'statusDetails'             => Creativestyle_AmazonCheckout_Model_Api_StatusDetails::class,
        'shippingAddress'           => Creativestyle_AmazonCheckout_Model_Api_Address::class,
        'billingAddress'            => Creativestyle_AmazonCheckout_Model_Api_Address::class,
        'constraints'               => array(Creativestyle_AmazonCheckout_Model_Api_Constraint::class),
        'deliverySpecifications'    => Creativestyle_AmazonCheckout_Model_Api_DeliverySpecifications::class,
        'providerMetadata'          => Creativestyle_AmazonCheckout_Model_Api_ProviderMetadata::class
    );

    /**
     * @var string
     */
    public $checkoutSessionId;

    /**
     * @var string
     */
    public $chargePermissionType;

    /**
     * @var Creativestyle_AmazonCheckout_Model_Api_WebCheckoutDetails
     */
    public $webCheckoutDetails;

    /**
     * @var string
     */
    public $productType;

    /**
     * @var Creativestyle_AmazonCheckout_Model_Api_PaymentDetails
     */
    public $paymentDetails;

    /**
     * @var Creativestyle_AmazonCheckout_Model_Api_MerchantMetadata
     */
    public $merchantMetadata;

    /**
     * @var string
     */
    public $platformId;

    /**
     * @var Creativestyle_AmazonCheckout_Model_Api_Buyer
     */
    public $buyer;

    /**
     * @var Creativestyle_AmazonCheckout_Model_Api_Address
     */
    public $shippingAddress;

    /**
     * @var Creativestyle_AmazonCheckout_Model_Api_Address
     */
    public $billingAddress;

    /**
     * @var Creativestyle_AmazonCheckout_Model_Api_PaymentPreference[]
     */
    public $paymentPreferences;

    /**
     * @var Creativestyle_AmazonCheckout_Model_Api_StatusDetails
     */
    public $statusDetails;

    /**
     * @var string
     */
    public $chargePermissionId;

    /**
     * @var string
     */
    public $chargeId;

    /**
     * @var Creativestyle_AmazonCheckout_Model_Api_Constraint[]
     */
    public $constraints;

    /**
     * @var string
     */
    public $creationTimestamp;

    /**
     * @var string
     */
    public $expirationTimestamp;

    /**
     * @var string
     */
    public $storeId;

    /**
     * @var Creativestyle_AmazonCheckout_Model_Api_DeliverySpecifications
     */
    public $deliverySpecifications;

    /**
     * @var Creativestyle_AmazonCheckout_Model_Api_ProviderMetadata
     */
    public $providerMetadata;

    /**
     * @var string
     */
    public $releaseEnvironment;

    /**
     * @inheritDoc
     */
    public function getCheckoutSessionId()
    {
        return $this->checkoutSessionId;
    }

    /**
     * @inheritDoc
     */
    public function getBuyerEmail() {
        if ($this->buyer) {
            return $this->buyer->email;
        }

        return null;
    }

    /**
     * @inheritDoc
     */
    public function getBillingAddress()
    {
        return $this->billingAddress;
    }

    /**
     * @inheritDoc
     */
    public function getState()
    {
        return $this->statusDetails->state;
    }

    /**
     * @inheritDoc
     */
    public function getShippingAddress()
    {
        return $this->shippingAddress;
    }

    /**
     * @inheritDoc
     */
    public function getPaymentDescription()
    {
        if (is_array($this->paymentPreferences) && !empty($this->paymentPreferences)) {
            return $this->paymentPreferences[0]->paymentDescriptor;
        }

        return null;
    }

    /**
     * @inheritDoc
     */
    public function getAmazonPayRedirectUrl()
    {
        return $this->webCheckoutDetails ? $this->webCheckoutDetails->amazonPayRedirectUrl : null;
    }

    /**
     * @inheritDoc
     */
    public function getChargePermissionId()
    {
        return $this->chargePermissionId;
    }
}
